@@ -163,40 +163,46 @@ void PlainTextEditor::contextMenuEvent(QContextMenuEvent *event)
         }
 
         if (!isReadOnly() && spellCheckingSupport()) {
-            QAction *spellCheckAction = popup->addAction(QIcon::fromTheme(QStringLiteral("tools-check-spelling")), i18n("Check Spelling..."), this, SLOT(slotCheckSpelling()));
-            if (emptyDocument) {
-                spellCheckAction->setEnabled(false);
+            if (!d->speller) {
+                d->speller = new Sonnet::Speller();
             }
-            popup->addSeparator();
-            QAction *autoSpellCheckAction = popup->addAction(i18n("Auto Spell Check"), this, SLOT(slotToggleAutoSpellCheck()));
-            autoSpellCheckAction->setCheckable(true);
-            autoSpellCheckAction->setChecked(checkSpellingEnabled());
-            popup->addAction(autoSpellCheckAction);
-
-            if (checkSpellingEnabled() &&  d->activateLanguageMenu) {
-                QMenu *languagesMenu = new QMenu(i18n("Spell Checking Language"), popup);
-                QActionGroup *languagesGroup = new QActionGroup(languagesMenu);
-                languagesGroup->setExclusive(true);
-                if (!d->speller) {
-                    d->speller = new Sonnet::Speller();
+            if (!d->speller->availableBackends().isEmpty()) {
+                QAction *spellCheckAction = popup->addAction(QIcon::fromTheme(QStringLiteral("tools-check-spelling")), i18n("Check Spelling..."), this, SLOT(slotCheckSpelling()));
+                if (emptyDocument) {
+                    spellCheckAction->setEnabled(false);
                 }
+                popup->addSeparator();
+                QAction *autoSpellCheckAction = popup->addAction(i18n("Auto Spell Check"), this, SLOT(slotToggleAutoSpellCheck()));
+                autoSpellCheckAction->setCheckable(true);
+                autoSpellCheckAction->setChecked(checkSpellingEnabled());
+                popup->addAction(autoSpellCheckAction);
 
-                QMapIterator<QString, QString> i(d->speller->availableDictionaries());
+                if (checkSpellingEnabled() &&  d->activateLanguageMenu) {
+                    QMenu *languagesMenu = new QMenu(i18n("Spell Checking Language"), popup);
+                    QActionGroup *languagesGroup = new QActionGroup(languagesMenu);
+                    languagesGroup->setExclusive(true);
 
-                while (i.hasNext()) {
-                    i.next();
+                    QMapIterator<QString, QString> i(d->speller->availableDictionaries());
 
-                    QAction *languageAction = languagesMenu->addAction(i.key());
-                    languageAction->setCheckable(true);
-                    languageAction->setChecked(spellCheckingLanguage() == i.value() || (spellCheckingLanguage().isEmpty()
-                                               && d->speller->defaultLanguage() == i.value()));
-                    languageAction->setData(i.value());
-                    languageAction->setActionGroup(languagesGroup);
-                    connect(languageAction, &QAction::triggered, this, &PlainTextEditor::slotLanguageSelected);
+                    QString defaultSpellcheckingLanguage = spellCheckingLanguage();
+                    if (defaultSpellcheckingLanguage.isEmpty()) {
+                        //TODO fix default value
+                        defaultSpellcheckingLanguage = d->speller->defaultLanguage();
+                    }
+                    while (i.hasNext()) {
+                        i.next();
+
+                        QAction *languageAction = languagesMenu->addAction(i.key());
+                        languageAction->setCheckable(true);
+                        languageAction->setChecked(defaultSpellcheckingLanguage == i.value());
+                        languageAction->setData(i.value());
+                        languageAction->setActionGroup(languagesGroup);
+                        connect(languageAction, &QAction::triggered, this, &PlainTextEditor::slotLanguageSelected);
+                    }
+                    popup->addMenu(languagesMenu);
                 }
-                popup->addMenu(languagesMenu);
+                popup->addSeparator();
             }
-            popup->addSeparator();
         }
         if (d->supportFeatures & TextToSpeech) {
             if (KPIMTextEdit::TextToSpeech::self()->isReady()) {
@@ -340,10 +346,15 @@ void PlainTextEditor::setReadOnly(bool readOnly)
 void PlainTextEditor::slotCheckSpelling()
 {
     if (document()->isEmpty()) {
-        KMessageBox::information(this, i18n("Nothing to spell check."));
+        slotDisplayMessageIndicator(i18n("Nothing to spell check."));
         return;
     }
     Sonnet::BackgroundChecker *backgroundSpellCheck = new Sonnet::BackgroundChecker;
+    if (backgroundSpellCheck->speller().availableBackends().isEmpty()) {
+        slotDisplayMessageIndicator(i18n("No backend available for spell checking."));
+        delete backgroundSpellCheck;
+        return;
+    }
     if (!d->spellCheckingLanguage.isEmpty()) {
         backgroundSpellCheck->changeLanguage(d->spellCheckingLanguage);
     }
@@ -443,7 +454,7 @@ bool PlainTextEditor::event(QEvent *ev)
     return QPlainTextEdit::event(ev);
 }
 
-bool PlainTextEditor::overrideShortcut(const QKeyEvent *event)
+bool PlainTextEditor::overrideShortcut(QKeyEvent *event)
 {
     const int key = event->key() | event->modifiers();
     if (KStandardShortcut::copy().contains(key)) {
@@ -486,11 +497,13 @@ bool PlainTextEditor::overrideShortcut(const QKeyEvent *event)
         return true;
     } else if (event->matches(QKeySequence::SelectAll)) { // currently missing in QTextEdit
         return true;
+    } else if (event == QKeySequence::DeleteEndOfLine) {
+        return true;
     }
     return false;
 }
 
-bool PlainTextEditor::handleShortcut(const QKeyEvent *event)
+bool PlainTextEditor::handleShortcut(QKeyEvent *event)
 {
     const int key = event->key() | event->modifiers();
 
@@ -603,6 +616,17 @@ bool PlainTextEditor::handleShortcut(const QKeyEvent *event)
             insertPlainText(text);    // TODO: check if this is html? (MiB)
         }
         return true;
+    } else if (event == QKeySequence::DeleteEndOfLine) {
+        QTextCursor cursor = textCursor();
+        QTextBlock block = cursor.block();
+        if (cursor.position() == block.position() + block.length() - 2) {
+            cursor.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
+        } else {
+            cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+        }
+        cursor.removeSelectedText();
+        setTextCursor(cursor);
+        return true;
     }
     return false;
 }
