@@ -46,6 +46,7 @@
 #include <QScrollBar>
 #include <QApplication>
 #include <QClipboard>
+#include <QDebug>
 
 using namespace KPIMTextEdit;
 class Q_DECL_HIDDEN RichTextEditor::RichTextEditorPrivate
@@ -185,40 +186,46 @@ QMenu *RichTextEditor::mousePopupMenu(QPoint pos)
         }
 
         if (!isReadOnly() && spellCheckingSupport()) {
-            QAction *spellCheckAction = popup->addAction(QIcon::fromTheme(QStringLiteral("tools-check-spelling")), i18n("Check Spelling..."), this, SLOT(slotCheckSpelling()));
-            if (emptyDocument) {
-                spellCheckAction->setEnabled(false);
+            if (!d->speller) {
+                d->speller = new Sonnet::Speller();
             }
-            popup->addSeparator();
-            QAction *autoSpellCheckAction = popup->addAction(i18n("Auto Spell Check"), this, SLOT(slotToggleAutoSpellCheck()));
-            autoSpellCheckAction->setCheckable(true);
-            autoSpellCheckAction->setChecked(checkSpellingEnabled());
-            popup->addAction(autoSpellCheckAction);
-
-            if (checkSpellingEnabled() &&  d->activateLanguageMenu) {
-                QMenu *languagesMenu = new QMenu(i18n("Spell Checking Language"), popup);
-                QActionGroup *languagesGroup = new QActionGroup(languagesMenu);
-                languagesGroup->setExclusive(true);
-                if (!d->speller) {
-                    d->speller = new Sonnet::Speller();
+            if (!d->speller->availableBackends().isEmpty()) {
+                QAction *spellCheckAction = popup->addAction(QIcon::fromTheme(QStringLiteral("tools-check-spelling")), i18n("Check Spelling..."), this, SLOT(slotCheckSpelling()));
+                if (emptyDocument) {
+                    spellCheckAction->setEnabled(false);
                 }
+                popup->addSeparator();
+                QAction *autoSpellCheckAction = popup->addAction(i18n("Auto Spell Check"), this, SLOT(slotToggleAutoSpellCheck()));
+                autoSpellCheckAction->setCheckable(true);
+                autoSpellCheckAction->setChecked(checkSpellingEnabled());
+                popup->addAction(autoSpellCheckAction);
 
-                QMapIterator<QString, QString> i(d->speller->availableDictionaries());
+                if (checkSpellingEnabled() &&  d->activateLanguageMenu) {
+                    QMenu *languagesMenu = new QMenu(i18n("Spell Checking Language"), popup);
+                    QActionGroup *languagesGroup = new QActionGroup(languagesMenu);
+                    languagesGroup->setExclusive(true);
 
-                while (i.hasNext()) {
-                    i.next();
+                    QMapIterator<QString, QString> i(d->speller->availableDictionaries());
 
-                    QAction *languageAction = languagesMenu->addAction(i.key());
-                    languageAction->setCheckable(true);
-                    languageAction->setChecked(spellCheckingLanguage() == i.value() || (spellCheckingLanguage().isEmpty()
-                                               && d->speller->defaultLanguage() == i.value()));
-                    languageAction->setData(i.value());
-                    languageAction->setActionGroup(languagesGroup);
-                    connect(languageAction, &QAction::triggered, this, &RichTextEditor::slotLanguageSelected);
+                    QString defaultSpellcheckingLanguage = spellCheckingLanguage();
+                    if (defaultSpellcheckingLanguage.isEmpty()) {
+                        defaultSpellcheckingLanguage = d->speller->defaultLanguage();
+                    }
+
+                    while (i.hasNext()) {
+                        i.next();
+
+                        QAction *languageAction = languagesMenu->addAction(i.key());
+                        languageAction->setCheckable(true);
+                        languageAction->setChecked(defaultSpellcheckingLanguage == i.value());
+                        languageAction->setData(i.value());
+                        languageAction->setActionGroup(languagesGroup);
+                        connect(languageAction, &QAction::triggered, this, &RichTextEditor::slotLanguageSelected);
+                    }
+                    popup->addMenu(languagesMenu);
                 }
-                popup->addMenu(languagesMenu);
+                popup->addSeparator();
             }
-            popup->addSeparator();
         }
 
         if (allowTabSupport() && !isReadOnly()) {
@@ -408,6 +415,11 @@ void RichTextEditor::checkSpelling(bool force)
         return;
     }
     Sonnet::BackgroundChecker *backgroundSpellCheck = new Sonnet::BackgroundChecker;
+    if (backgroundSpellCheck->speller().availableBackends().isEmpty()) {
+        slotDisplayMessageIndicator(i18n("No backend available for spell checking."));
+        delete backgroundSpellCheck;
+        return;
+    }
     if (!d->spellCheckingLanguage.isEmpty()) {
         backgroundSpellCheck->changeLanguage(d->spellCheckingLanguage);
     }
@@ -671,7 +683,7 @@ bool RichTextEditor::event(QEvent *ev)
     return QTextEdit::event(ev);
 }
 
-bool RichTextEditor::handleShortcut(const QKeyEvent *event)
+bool RichTextEditor::handleShortcut(QKeyEvent *event)
 {
     const int key = event->key() | event->modifiers();
 
@@ -784,11 +796,23 @@ bool RichTextEditor::handleShortcut(const QKeyEvent *event)
             insertPlainText(text);    // TODO: check if this is html? (MiB)
         }
         return true;
+    } else if (event == QKeySequence::DeleteEndOfLine) {
+        QTextCursor cursor = textCursor();
+        QTextBlock block = cursor.block();
+        if (cursor.position() == block.position() + block.length() - 2) {
+            cursor.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
+        } else {
+            cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+        }
+        cursor.removeSelectedText();
+        setTextCursor(cursor);
+        return true;
     }
+
     return false;
 }
 
-bool RichTextEditor::overrideShortcut(const QKeyEvent *event)
+bool RichTextEditor::overrideShortcut(QKeyEvent *event)
 {
     const int key = event->key() | event->modifiers();
 
@@ -832,6 +856,8 @@ bool RichTextEditor::overrideShortcut(const QKeyEvent *event)
         return true;
     } else if (event->matches(QKeySequence::SelectAll)) { // currently missing in QTextEdit
         return true;
+    } else if (event == QKeySequence::DeleteEndOfLine) {
+        return true;
     }
     return false;
 }
